@@ -19,34 +19,15 @@
  *     Jasper St. Pierre <jstpierre@mecheye.net>
  */
 
-#ifndef __GIS_ASSISTANT_H__
-#define __GIS_ASSISTANT_H__
+#pragma once
 
 #include "gis-page.h"
 
 G_BEGIN_DECLS
 
 #define GIS_TYPE_ASSISTANT               (gis_assistant_get_type ())
-#define GIS_ASSISTANT(obj)                           (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIS_TYPE_ASSISTANT, GisAssistant))
-#define GIS_ASSISTANT_CLASS(klass)                   (G_TYPE_CHECK_CLASS_CAST ((klass),  GIS_TYPE_ASSISTANT, GisAssistantClass))
-#define GIS_IS_ASSISTANT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GIS_TYPE_ASSISTANT))
-#define GIS_IS_ASSISTANT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GIS_TYPE_ASSISTANT))
-#define GIS_ASSISTANT_GET_CLASS(obj)                 (G_TYPE_INSTANCE_GET_CLASS ((obj),  GIS_TYPE_ASSISTANT, GisAssistantClass))
 
-typedef struct _GisAssistant        GisAssistant;
-typedef struct _GisAssistantClass   GisAssistantClass;
-
-struct _GisAssistant
-{
-  GtkBox parent;
-};
-
-struct _GisAssistantClass
-{
-  GtkBoxClass parent_class;
-};
-
-GType gis_assistant_get_type (void);
+G_DECLARE_FINAL_TYPE (GisAssistant, gis_assistant, GIS, ASSISTANT, GtkBox)
 
 void      gis_assistant_add_page          (GisAssistant *assistant,
                                            GisPage      *page);
@@ -63,5 +44,3 @@ gboolean  gis_assistant_save_data         (GisAssistant  *assistant,
                                            GError       **error);
 
 G_END_DECLS
-
-#endif /* __GIS_ASSISTANT_H__ */
